// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


/*! \file
	This header defines utility functions to deal with complex numbers.*/

/*!	Returns the complex conjugate of the given complex number (i.e. it changes 
	the sign of the y-component).*/
float2 conjugate(float2 z){
	return float2(z.x,-z.y);
}
/*!	This function implements complex multiplication.*/
float2 multiply(float2 lhs,float2 rhs){
	return float2(lhs.x*rhs.x-lhs.y*rhs.y,lhs.x*rhs.y+lhs.y*rhs.x);
}
/*!	This function computes the squared magnitude of the given complex number.*/
float absSqr(float2 z){
	return dot(z,z);
}
/*!	This function computes the quotient of two complex numbers. The denominator 
	must not be zero.*/
float2 divide(float2 numerator,float2 denominator){
	return float2(numerator.x*denominator.x+numerator.y*denominator.y,-numerator.x*denominator.y+numerator.y*denominator.x)/dot(denominator,denominator);
}
/*!	This function implements computation of the reciprocal of the given 
	non-zero complex number.*/
float2 reciprocal(float2 z){
	return float2(z.x,-z.y)/dot(z,z);
}
